/********************************************************************************************************
 * @file     aaa_24G_RF.c
 *
 * @brief    for TLSR chips
 *
 * @author	 telink
 * @date     Sep. 30, 2010
 *
 * @par      Copyright (c) 2010, Telink Semiconductor (Shanghai) Co., Ltd.
 *           All rights reserved.
 *
 *			 The information contained herein is confidential and proprietary property of Telink
 * 		     Semiconductor (Shanghai) Co., Ltd. and is available under the terms
 *			 of Commercial License Agreement between Telink Semiconductor (Shanghai)
 *			 Co., Ltd. and the licensee in separate contract or the terms described here-in.
 *           This heading MUST NOT be removed from this file.
 *
 * 			 Licensees are granted free, non-transferable use of the information in this
 *			 file under Mutual Non-Disclosure Agreement. NO WARRENTY of ANY KIND is provided.
 *
 *******************************************************************************************************/

#include "aaa_public_config.h"

#if(CLOCK_SYS_CLOCK_HZ == 32000000 || CLOCK_SYS_CLOCK_HZ == 24000000)
    #define			SHIFT_US		5
    #elif(CLOCK_SYS_CLOCK_HZ == 16000000 || CLOCK_SYS_CLOCK_HZ == 12000000)
    #define			SHIFT_US		4
    #elif(CLOCK_SYS_CLOCK_HZ == 8000000)
    #define			SHIFT_US		3
#else
   #define			SHIFT_US		6
#endif

task_when_rf_func p_task_when_rf = NULL;

#define  RX_FRAME_SIZE     64
_attribute_data_retention_user volatile unsigned char  rx_packet[RX_FRAME_SIZE]  __attribute__((aligned(4)));
_attribute_data_retention_user volatile unsigned char ack_payload[RX_FRAME_SIZE];
_attribute_data_retention_user volatile  unsigned int rptr = 0;
_attribute_data_retention_user volatile unsigned int  rf_state = 0;

#if TEST_LOST_RATE
    volatile unsigned int  rf_rx_cnt = 0;
    volatile unsigned int  rf_rx_timeout_cnt = 0;
    volatile unsigned int  user_rx_timeout_cnt = 0;
    volatile unsigned int	user_rx_right_data_cnt = 0;
#endif

_attribute_data_retention_user u8 chn_mask = 0x80;

#ifndef			CHANNEL_SLOT_TIME
    #define			CHANNEL_SLOT_TIME			8000
#endif

#ifndef		PKT_BUFF_SIZE
    #define		PKT_BUFF_SIZE		64
#endif

#define		LL_CHANNEL_SYNC_TH				2
#define		LL_CHANNEL_SEARCH_TH			60
#define		LL_CHANNEL_SEARCH_FLAG			BIT(16)
#define		LL_NEXT_CHANNEL(c)				((c + 6) & 14)

//_attribute_data_retention_user unsigned char  rf_rx_buff[PKT_BUFF_SIZE*2] __attribute__((aligned(4)));
//_attribute_data_retention_user int		rf_rx_wptr;

_attribute_data_retention_user u8		device_channel = 0;
_attribute_data_retention_user u8		ll_chn_sel;
_attribute_data_retention_user u8		ll_rssi;

_attribute_data_retention_user u16		ll_chn_tick;
_attribute_data_retention_user u32		ll_chn_rx_tick;
_attribute_data_retention_user u32		ll_chn_mask = LL_CHANNEL_SEARCH_FLAG;
_attribute_data_retention_user u32		ll_clock_time;
_attribute_data_retention_user int		device_packet_received;

_attribute_data_retention_user volatile int	device_ack_received = 0;

_attribute_data_retention_user int		device_sync = 0;
//-------------------- rf init------------------
#define FRE_OFFSET 	0
#define FRE_STEP 	5
#define MAX_RF_CHANNEL  16
const unsigned char rf_chn[MAX_RF_CHANNEL] =
{
    FRE_OFFSET + 5, FRE_OFFSET + 9, FRE_OFFSET + 13, FRE_OFFSET + 17,
    FRE_OFFSET + 22, FRE_OFFSET + 26, FRE_OFFSET + 30, FRE_OFFSET + 35,
    FRE_OFFSET + 40, FRE_OFFSET + 45, FRE_OFFSET + 50, FRE_OFFSET + 55,
    FRE_OFFSET + 60, FRE_OFFSET + 65, FRE_OFFSET + 70, FRE_OFFSET + 76,
};

//use 8 channel ,  0-->6-->12-->2-->8--->14-->4-->10-->0

#if 0
#define	   	FOR_NORDIC	1
const TBLCMDSET tbl_rf_init_aaa[] =
{
    {0x12d2, 0x9b,  TCMD_UNDER_BOTH | TCMD_WRITE}, //DCOC_DBG0
    {0x12d3, 0x19,  TCMD_UNDER_BOTH | TCMD_WRITE}, //DCOC_DBG1
    {0x127b, 0x0e,  TCMD_UNDER_BOTH | TCMD_WRITE}, //BYPASS_FILT_1
    {0x1276, 0x50,  TCMD_UNDER_BOTH | TCMD_WRITE}, //FREQ_CORR_CFG2_0
    {0x1277, 0x73,  TCMD_UNDER_BOTH | TCMD_WRITE}, //FREQ_CORR_CFG2_1
};

const TBLCMDSET  tbl_rf_private_2m_aaa[] =
{
    {0x1220, 0x04, TCMD_UNDER_BOTH | TCMD_WRITE},//SC_CODE
    {0x1221, 0x2a, TCMD_UNDER_BOTH | TCMD_WRITE},//IF_FREQ
    {0x1222, 0x43, TCMD_UNDER_BOTH | TCMD_WRITE},//HPMC_EXP_DIFF_COUNT_L
    {0x1223, 0x06, TCMD_UNDER_BOTH | TCMD_WRITE},//HPMC_EXP_DIFF_COUNT_H
    {0x1254, 0x0e,  TCMD_UNDER_BOTH | TCMD_WRITE}, //AGC_THRSHLD1_2M_0
    {0x1255, 0x09,  TCMD_UNDER_BOTH | TCMD_WRITE}, //AGC_THRSHLD1_2M_1
    {0x1256, 0x0c,  TCMD_UNDER_BOTH | TCMD_WRITE}, //AGC_THRSHLD2_2M_0
    {0x1257, 0x08,  TCMD_UNDER_BOTH | TCMD_WRITE}, //AGC_THRSHLD2_2M_1
    {0x1258, 0x09,  TCMD_UNDER_BOTH | TCMD_WRITE}, //AGC_THRSHLD3_2M_0
    {0x1259, 0x0f,  TCMD_UNDER_BOTH | TCMD_WRITE}, //AGC_THRSHLD3_2M_1

    {0x400, 0x1f, TCMD_UNDER_BOTH | TCMD_WRITE},//tx mode
    {0x402, 0x44, TCMD_UNDER_BOTH | TCMD_WRITE},//preamble length
    {0x404, 0xca, TCMD_UNDER_BOTH | TCMD_WRITE},//nordic head
    {0x421, 0xa1, TCMD_UNDER_BOTH | TCMD_WRITE},//len_0_en
    //AGC table
    {0x460, 0x36, TCMD_UNDER_BOTH | TCMD_WRITE},//grx_0
    {0x461, 0x46, TCMD_UNDER_BOTH | TCMD_WRITE},//grx_1
    {0x462, 0x51, TCMD_UNDER_BOTH | TCMD_WRITE},//grx_2
    {0x463, 0x61, TCMD_UNDER_BOTH | TCMD_WRITE},//grx_3
    {0x464, 0x6d, TCMD_UNDER_BOTH | TCMD_WRITE},//grx_4
    {0x465, 0x78, TCMD_UNDER_BOTH | TCMD_WRITE},//grx_5
};
#endif

_attribute_ram_code_ void rf_drv_private_2m_init()
{
    //tbl_rf_init
    write_reg8(0x12d2, 0x9b);//DCOC_DBG0
    write_reg8(0x12d3, 0x19); //DCOC_DBG1
    write_reg8(0x127b, 0x0e); //BYPASS_FILT_1
    write_reg8(0x1279, 0x38);

////start  add follow setting follow 8278 drv
    write_reg8(0x124a, 0x0e);	//POW_001_010_L
    write_reg8(0x124b, 0x09); 	//POW_001_010_H
    write_reg8(0x124e, 0x09); 	//POW_101_100_L
    write_reg8(0x124f, 0x0f);	//POW_101_100_H
    write_reg8(0x1254, 0x0e);	//POW_001_010_L
    write_reg8(0x1255, 0x09); 	//POW_001_010_H
    write_reg8(0x1256, 0x0c); 	//POW_011_100_L
    write_reg8(0x1257, 0x08);	//POW_011_100_H
    write_reg8(0x1258, 0x09);	//POW_101_100_L
    write_reg8(0x1259, 0x0f); 	//POW_101_100_H
    write_reg8(0x1276, 0x50); 	//FREQ_CORR_CFG2_0	
    write_reg8(0x1277, 0x73); 	//FREQ_CORR_CFG2_1
    //For optimum Rx chain	
    write_reg8(0x134e, 0x45); //CBPF_TRIM_I && CBPF_TRIM_Q
    write_reg8(0x134c, 0x02); //LNA_ITRIM=0x01(default)(change to 0x02[TBD])
////end add 

  //tbl_rf_private_2m
    write_reg8(0x1220, 0x04);//SC_CODE
    write_reg8(0x1221, 0x2a);//IF_FREQ
    write_reg8(0x1222, 0x43);//HPMC_EXP_DIFF_COUNT_L
    write_reg8(0x1223, 0x06);//HPMC_EXP_DIFF_COUNT_H
    write_reg8(0x1254, 0x0e); //AGC_THRSHLD1_2M_0
    write_reg8(0x1255, 0x09); //AGC_THRSHLD1_2M_1
    write_reg8(0x1256, 0x0c); //AGC_THRSHLD2_2M_0
    write_reg8(0x1257, 0x08); //AGC_THRSHLD2_2M_1
    write_reg8(0x1258, 0x09); //AGC_THRSHLD3_2M_0
    write_reg8(0x1259, 0x0f); //AGC_THRSHLD3_2M_1

    write_reg8(0x400, 0x1f);//tx mode
    write_reg8(0x402, 0x44);//preamble length
    write_reg8(0x404, 0xea);//nordic head
    write_reg8(0x421, 0xa1);//len_0_en
    write_reg8(0x430, 0x3c);//<1>hd_timestamp
    //AGC table
    write_reg8(0x460, 0x36);//grx_0
    write_reg8(0x461, 0x46);//grx_1
    write_reg8(0x462, 0x51);//grx_2
    write_reg8(0x463, 0x61);//grx_3
    write_reg8(0x464, 0x6d);//grx_4
    write_reg8(0x465, 0x78);//grx_5
    
////Add start the following settings compared with tbl_rf_zigbee_250k and these settings are default values.
    write_reg8(0x122a, 0x90);
    write_reg8(0x122c, 0x38);
    write_reg8(0x123d, 0x00);
    write_reg8(0x420, 0x1e);
    write_reg8(0x405, 0x04);
    write_reg8(0x422, 0x04);
    write_reg8(0x408, 0xc9);
    write_reg8(0x409, 0x8a);
    write_reg8(0x40a, 0x11);
    write_reg8(0x40b, 0xf8);
    write_reg8(0x407, 0x01);
    write_reg8(0x403, 0x44);
//// add end

    write_reg16(0xf06, 0);//rx wait time
    write_reg8(0x0f0c, 0x54); //rx settle time: Default 150us, minimum 85us(0x54+1)

    write_reg16(0xf0e, 0);//tx wait time
    //write_reg8(0x0f04, 0x70);//tx settle time: Default 150us, minimum 113us(0x70+1)

    write_reg8(0xf10, 0);// wait time on NAK

    reg_dma_chn_en |= FLD_DMA_CHN_RF_RX  | FLD_DMA_CHN_RF_TX;

    //g_RFMode = RF_MODE_PRIVATE_2M;

    rf_rx_buffer_set((u8 *)rx_packet, RX_FRAME_SIZE, 0);

    //rf_acc_len_set(5);
    write_reg8(0x405, (read_reg8(0x405)&0xf8) |5); //access_byte_num[2:0]
    write_reg8(0x420, 38);
	
    rf_set_tx_rx_off();

    rf_rx_acc_code_enable(0x01);
    rf_tx_acc_code_select(0);

    irq_disable();
    irq_clr_src();
    rf_irq_src_allclr();
    irq_enable_type(FLD_IRQ_ZB_RT_EN); //enable RF irq
    rf_irq_disable(FLD_RF_IRQ_ALL);
    rf_irq_enable(FLD_RF_IRQ_TX | FLD_RF_IRQ_RX | FLD_RF_IRQ_RX_TIMEOUT); //
    irq_enable();
}
#if 0
static inline void rf_set_tx_rx_off_auto_mode(void)
{
    write_reg8(0xf00, 0x80);
}
#endif
void rf_start_stxtorx(void *addr, unsigned int tick, unsigned int timeout_us)
{
    write_reg16(0x800f0a, timeout_us - 1);
    write_reg32(0x800f18, tick);
    write_reg8(0x800f16, read_reg8(0x800f16) | 0x04);	 // Enable cmd_schedule mode
    write_reg16(0x800c0c, (unsigned short)((unsigned int)addr));
    write_reg8(0x800f00, 0x87);	   // single tx2rx
}

void rf_set_access_code0(u32 code)
{
    write_reg32(0x800408, (read_reg32(0x800408) & 0xff) | (code & 0xffffff00));
    write_reg8(0x80040c, code);
}
void set_pair_access_code(u32 code)
{
    write_reg32(0x800408, ((code & 0xffffff00) | 0x71));
    write_reg8(0x80040c, code);
}
void set_data_access_code(u32 code)
{
    write_reg32(0x800408, ((code & 0xffffff00) | 0x77));
    write_reg8(0x80040c, code);
}

void rf_set_access_code1(u32 code)
{
    write_reg32(0x800410, (read_reg32(0x800410) & 0xff) | (code & 0xffffff00));
    write_reg8(0x800414, code);
}
u32 rf_get_access_code1()
{
    return read_reg8(0x800414) | (read_reg32(0x800410) & 0xffffff00);
}
void rf_irq_src_allclr()
{
    reg_rf_irq_status = 0xffff;
}

u8  Read_Payload(u8 *payload)
{
    u8 *rx_pkt;
    u8 ret, i;
    if (NULL == payload)
        return 0;

    rx_pkt = (u8 *) rx_packet ;

    rf_rx_buffer_set((u8 *)rx_packet, RX_FRAME_SIZE, 0);

    ret = rx_pkt[4] & 0x3f; //get length of rx paylaod

    for (i = 0; i < (ret + 5); i++)
    {
        payload[i] = rx_pkt[i];
    }
    return 1;
}

_attribute_ram_code_ void irq_handle_private_2m()
{
    unsigned short rf_irq_src = rf_irq_src_get();
    rf_state = rf_irq_src;

    if (rf_irq_src)
    {
        if (rf_irq_src & FLD_RF_IRQ_RX)
        {
            rf_irq_clr_src(FLD_RF_IRQ_RX);
#if TEST_LOST_RATE
            rf_rx_cnt++;
#endif
            DEBUG_B2_PIN_LOW;
        }

        if (rf_irq_src & FLD_RF_IRQ_TX)
        {
            rf_irq_clr_src(FLD_RF_IRQ_TX);
            DEBUG_D0_PIN_LOW;
        }

        if (rf_irq_src & FLD_RF_IRQ_RX_TIMEOUT)
        {
            rf_irq_clr_src(FLD_RF_IRQ_RX_TIMEOUT);
            DEBUG_D1_PIN_LOW;
#if TEST_LOST_RATE
            rf_rx_timeout_cnt++;
#endif
        }
        irq_clr_src();
    }
}

#if 0
u8	get_next_channel_with_mask(u32 mask, u8 chn)
{
    int chn_high = (mask >> 4) & 0x0f;

    if (mask & LL_CHANNEL_SEARCH_FLAG)	
    {
        return LL_NEXT_CHANNEL(chn);	
    }
    else if (chn_high != chn)
    {
        ll_chn_sel = 1;
        return chn_high;
    }
    else
    {
        ll_chn_sel = 0;
        return mask & 0x0f;	
    }
}
#else
_attribute_ram_code_  u8 get_next_channel_with_mask(u32 mask, u8 chn)
{
	static u8 flag=0;
	flag^=0xff;
   //RF_CHN_CHANGE_PIN_TOG(flag);
   return (chn+7)%21;

}
#endif

extern u8  rf_rx_process(rf_packet_ack_pairing_t *p_pkt);

_attribute_ram_code_ void irq_device_rx(void)
{
    rf_packet_ack_pairing_t *p = (rf_packet_ack_pairing_t *)(&ack_payload[0]);
	
    if (rf_rx_process(p) && ll_chn_tick != p->tick)
    {
        ll_chn_tick = p->tick;	

        device_sync = 1;
        device_ack_received = 1;

        ll_chn_mask = p->chn;
        ll_chn_rx_tick = clock_time();

#if TEST_LOST_RATE
        user_rx_right_data_cnt++;
#endif
    }
}

_attribute_ram_code_ int device_send_packet(u8 *p, int retry, u16 timeout)
{

   //// while (!clock_time_exceed(wakeup_next_tick, 500));
    //u8 rx_flag=0;

    _attribute_data_retention_user  static  u32 ack_miss_no;

    device_ack_received = 0;

    int i;
    //int step = 1;

    //if(device_sync)
    //{
     	//step = retry + 1;		// step ==4
    //}

    u32 tick = clock_time();
    //rf_power_enable (1);	
    for (i = 0; i <= retry; i += 1)
    {
        u32 t = clock_time();
        rf_state = 0;
        DEBUG_D1_PIN_HIGH;
        DEBUG_D0_PIN_HIGH;
        DEBUG_B2_PIN_HIGH;
#if 1
        rf_trx_state_set(RF_MODE_AUTO, rf_chn[device_channel]);
#else
        rf_trx_state_set(RF_MODE_AUTO, 59);
#endif
        rf_start_stxtorx(p, ClockTime(), timeout);

#if DO_TASK_WHEN_RF_IN_24G_MODE_ENALBE
        if (p_task_when_rf != NULL)
        {
            (*p_task_when_rf)();
            p_task_when_rf = NULL;
        }
#endif

        while (1)
        {
            if(rf_state & (FLD_RF_IRQ_RX))
            {
                if(Read_Payload((u8 *)(&ack_payload[0])))
                {
                    irq_device_rx();
                }
                break;
            }
		
            if(rf_state & (FLD_RF_IRQ_RX_TIMEOUT))
            {
                break;
            }

            if(clock_time_exceed(t, timeout + 550))
            {
		  #if TEST_LOST_RATE
                user_rx_timeout_cnt++;
              #endif
                break;
            }
        }
		
	  static u8 cnt=0;
        if (device_ack_received)	
        {
            ack_miss_no = 0;
		cnt=0;
            break;
        }
	
	  cnt++;
        ack_miss_no ++;
        if (ack_miss_no >= LL_CHANNEL_SEARCH_TH)
        {
            device_sync = 0;	
            ll_chn_mask = LL_CHANNEL_SEARCH_FLAG;
        }
        else if (ack_miss_no >= LL_CHANNEL_SYNC_TH)
        {
            device_sync = 0;
        }

        if (!device_sync)
        {
            cnt=0;
            device_channel = get_next_channel_with_mask(ll_chn_mask, device_channel);
        }
    }

#if 1
    if (device_ack_received)
    {
       static u8 t8ms_cnt=0;
	    u32 t1 = (ll_chn_rx_tick  - tick)/48;  //timeout * (i - 1);
	    
	   u8 host_cnt=(ll_chn_tick >>8);
	   if(t8ms_cnt!=host_cnt)
	   {
		   t8ms_cnt=host_cnt;
		   {
			   
			   //device_channel = get_next_channel_with_mask(ll_chn_mask, device_channel);
		   }
	   }
       
    }
    return device_ack_received;
#else
    _attribute_data_retention_user static int adjust = 0;
    int ret = 0;

    if(i <= retry)
    {
        //sync timing
        u32 t0 = (ll_chn_tick & 0xff) * 32;
        u32 t1 = (ll_chn_rx_tick  - tick) >> SHIFT_US;

        if (adjust)	
        {
            adjust = 0;
        }
        else if (t0 < t1)
        {
            adjust =  3000;
        }
        else
        {
            t1 = t0 - t1;
            if (t1 > 3000) 
            {
                adjust = -1000;
            }
            else if (t1 < 1000)
            {
                adjust = 1000;
            }
        }
        ret = 1;
    }
    else
    {
        adjust = 0;
    }
	
    wakeup_next_tick = wakeup_next_tick + adjust * 16;
    //cpu_adjust_system_tick (adjust); 	 //adjust 1ms

    return ret;
#endif
}
