/********************************************************************************************************
 * @file     aaa_main.c
 *
 * @brief    for TLSR chips
 *
 * @author	 telink
 * @date     Sep. 30, 2010
 *
 * @par      Copyright (c) 2010, Telink Semiconductor (Shanghai) Co., Ltd.
 *           All rights reserved.
 *
 *			 The information contained herein is confidential and proprietary property of Telink
 * 		     Semiconductor (Shanghai) Co., Ltd. and is available under the terms
 *			 of Commercial License Agreement between Telink Semiconductor (Shanghai)
 *			 Co., Ltd. and the licensee in separate contract or the terms described here-in.
 *           This heading MUST NOT be removed from this file.
 *
 * 			 Licensees are granted free, non-transferable use of the information in this
 *			 file under Mutual Non-Disclosure Agreement. NO WARRENTY of ANY KIND is provided.
 *
 *******************************************************************************************************/

#include "AAA_public_config.h"
#include "vendor/common/blt_common.h"

_attribute_data_retention_user u8 report_rate=8;			// ms
_attribute_data_retention_user u8 last_report_rate=0xff;	// ms

_attribute_data_retention_user int dev_info_idx;
_attribute_data_retention_user u8 pub_key[16] =
{
    0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08, 0x09, 0x0A, 0x0B, 0x0C, 0x0D, 0x0E, 0x0F, 0x10
};


_attribute_data_retention_user OUTPUT_DEV_INFO_AAA output_dev_info =
{
    0,//u32 bin_crc;
    {2, 4, 4}, //u8
    0,//u8 sensor_type;
    0,//u8 sensor_pd1;
    0,//u8 sensor_pd2;
    0,//u8 sensor_pd3;
    MOUSE_DATA_LEN_AAA,//u8 ms_data_len;
};

unsigned int clock_sys_hz = 16000000;
 
bool o_sendor_flag;

_attribute_ram_code_ void irq_handle_usb_timer()
{
    unsigned int irq_src = irq_get_src();
     unsigned short rf_irq_src = rf_irq_src_get();

    if(irq_src & FLD_IRQ_TMR1_EN)               //  1ms ok
    {
        if(reg_tmr_sta & FLD_TMR_STA_TMR1)
        {
            if(!o_sendor_flag)
                o_sendor_flag =1;
            //gpio_toggle(GPIO_PA0);
            reg_tmr_sta = FLD_TMR_STA_TMR1; //clear irq status
        }
    }
}

_attribute_ram_code_ void irq_handler(void)
{
    if (fun_mode == RF_2M_2P4G_MODE)
    {
        irq_handle_private_2m();
    }
    else if(fun_mode == RF_1M_BLE_MODE)
    {
        irq_blt_sdk_handler();
    }
    else if(fun_mode == USB_MODE)
    {
        irq_handle_usb_timer();
    }
}

//len must 4 bytes bei beisu  beacuse *(volatile u32 *)(s_addr + idx) must 4 bytes bei beisu
int flash_info_load_aaa(u32 s_addr, u8 *d_addr,  u8 len)
{
    int idx;
    u32 buf;
    for (idx = 0; idx < 4096; idx += len)
    {
        flash_read_page((u32)(s_addr + idx), 4, (u8 *)(&buf));
        if (buf == U32_MAX)
        {
            break;
        }
    }
    idx -= len;
    if (idx < 0) 		// no binding
    {
        return idx;
    }
    flash_read_page((u32)(s_addr + idx), len, d_addr);

    if (idx > 4000) 			//3k, erase flash
    {
        flash_erase_sector((u32)s_addr);

        sleep_us(10);

        flash_write_page((u32)s_addr, len, d_addr);
        idx = 0;
    }
    return idx;
}

#if IS_SINGLE_GPIO_CHANGE_MODE

void three_switch_mode_change()
{

    //gpio_setup_up_down_resistor(PIN_MODE_SWITCH,PM_PIN_PULLUP_1M);
    //sleep_us(20);
    if (IS_BLE_MODE_AAA)
    {
        flash_dev_info.mode = RF_1M_BLE_MODE;
        //gpio_set_input_en(GPIO_MODE_SWITCH,0);

    }
    else
    {
        flash_dev_info.mode = RF_2M_2P4G_MODE;
    }
    //gpio_setup_up_down_resistor(PIN_MODE_SWITCH,PM_PIN_UP_DOWN_FLOAT);
#if 0 //test switch is good
    gpio_set_output_en(PIN_BLE_LED, 1);
    while (1)
    {
        if (IS_BLE_MODE_AAA)
        {

            gpio_write(PIN_BLE_LED, LED_ON_AAA);
        }
        else
        {
            gpio_write(PIN_24G_LED, LED_ON_AAA);
        }
    }

#endif
}
#endif
void user_config_ykq()
{
    random_generator_init();  //this is must
    flash_read_page(CFG_ADR_MAC, sizeof(custom_cfg_t), (u8 *)&user_cfg.dev_mac);
    if (user_cfg.dev_mac == U32_MAX)
    {
        generateRandomNum(4, (u8 *) &user_cfg.dev_mac);
        flash_write_page(CFG_ADR_MAC, 4, (u8 *)&user_cfg.dev_mac);
    }


    if (user_cfg.paring_tx_power == U8_MAX)
    {
        user_cfg.paring_tx_power = DEFAULT_PAIR_TX_POWER;
    }
    if (user_cfg.tx_power == U8_MAX)
    {
        user_cfg.tx_power = DEFAULT_NORMAL_TX_POWER;
    }
    if (user_cfg.emi_tx_power == U8_MAX)
    {
        user_cfg.emi_tx_power = DEFAULT_EMI_TX_POWER;
    }


    if ((user_cfg.sensor_direct == U8_MAX))
    {
		
        user_cfg.sensor_direct = SENSOR_DIRECTION_CLOCK_6;
    }
    else if (user_cfg.sensor_direct == 0)
    {
        user_cfg.sensor_direct = SENSOR_DIRECTION_CLOCK_3;
    }
    else if (user_cfg.sensor_direct == 1)
    {
        user_cfg.sensor_direct = SENSOR_DIRECTION_CLOCK_6;
    }
    else if (user_cfg.sensor_direct == 2)
    {
        user_cfg.sensor_direct = SENSOR_DIRECTION_CLOCK_9;
    }
    else if (user_cfg.sensor_direct == 3)
    {
        user_cfg.sensor_direct = SENSOR_DIRECTION_CLOCK_12;
    }
     //if(user_cfg.internal_cap!=0)
     {
	 	//blc_app_setExternalCrystalCapEnable(1);
     }
    //aes Key
    for (u8 i = 0; i < 16; i++)
    {
        if (user_cfg.pub_key[i] != 0xff)
        {
            memcpy(pub_key, &user_cfg.pub_key[0], 16);
            break;
        }
    }

    for (u8 i = 0; i < 18; i++)
    {
        if (user_cfg.device_name[i] != 0xff)
        {
            device_name_len++;
        }
        else
        {
            break;
        }
    }

}

int deepRetWakeUp;

void powerLED(u32 pin, u8 inx)
{
	gpio_set_output_en(pin, 1);
	for(int i = 0; i < inx; ++i)
	{
		gpio_write(pin, 1);
		cpu_sleep_wakeup(SUSPEND_MODE, PM_WAKEUP_TIMER, clock_time()+ 300*CLOCK_SYS_CLOCK_1MS);
		gpio_write(pin, 0);
		cpu_sleep_wakeup(SUSPEND_MODE, PM_WAKEUP_TIMER, clock_time()+ 300*CLOCK_SYS_CLOCK_1MS);
	}
	gpio_set_output_en(pin, 0);
}

void usb_user_init()
{
	//enable USB manual interrupt(in auto interrupt mode,USB device would be USB printer device)
	usb_init_interrupt();
	
#if USB_DESCRIPTOR_MY_SELF
	write_reg8(0x10e,(1<<USB_EDP_MOUSE)|(1<<USB_EDP_KEYBOARD_IN)|(1<<USB_EDP_SPP_IN)|(1<<USB_EDP_SPP_OUT));
#endif
	deepsleep_dp_dm_gpio_low_wake_enable();

	//enable USB DP pull up 1.5k
	usb_set_pin_en();
	connect_ok =1;
}

_attribute_ram_code_ void ui_opt_sensor()
{
    if (OPTSensor_motion_report(0))
    {
        has_new_key_event |= SENSOR_DATA_EVENT_AAA;
     	 has_new_report_aaa |= HAS_MOUSE_REPORT;
        mouse_xy_multiple();
        check_sensor_dircet(user_cfg.sensor_direct);
        adaptive_smoother();
    }
    else
    {
        ms_data.x = 0;
        ms_data.y = 0;
    }
}

_attribute_ram_code_ void ui_usb_mode()
{
	static u32 tick_usb_loop;

    if(o_sendor_flag)
    {
        //gpio_toggle(GPIO_PA0);
        ui_opt_sensor();
        o_sendor_flag =0;
    }
	
	if(clock_time_exceed(tick_usb_loop,8000))
	{
		get_usb_data_report_aaa();
		tick_usb_loop=clock_time();
	}
	
	if(has_new_report_aaa)
	{
		if(usb_mouse_hid_report_aaa(1, &ms_data.btn, sizeof(mouse_data_t)))
		{
			has_new_report_aaa=0;
		}
	} 
}

u8 allow_suspend = 0;
u8 need_wake_up_host_flag;
u32 tick_resume_host;
u32 tick_host_start_power_off=0;

void usb_proc_suspend (void)
{
	u8 power_off_flag=0;
	
	if(reg_irq_src & FLD_IRQ_USB_PWDN_EN)
	{
		power_off_flag=1;
	}
	
	if(power_off_flag)
	{
		if(tick_host_start_power_off)
		{
			if(clock_time_exceed(tick_host_start_power_off, 1000000))
			{
				tick_host_start_power_off=0;
				tick_resume_host=clock_time()+1000000*16;
				allow_suspend=1;
			}
			else
			{
				need_wake_up_host_flag=0;
			}
		}
		else if(need_wake_up_host_flag)
		{
			if(tick_resume_host==0)
			{
				tick_resume_host=clock_time()|1;
			}
			if(clock_time_exceed(tick_resume_host, 200000))
			{
				tick_resume_host=0;
				usb_resume_host();
				allow_suspend = 0;
			}	
		}
		else if(allow_suspend)
		{
			#if WHEEL_FUN_ENABLE_AAA
	            		wheel_set_wakeup_level_suspend(1);
			#endif
			
			#if BUTTON_FUN_ENABLE_AAA
	            		btn_set_wakeup_level_suspend(1);
			#endif
			
			#if SENSOR_FUN_ENABLE_AAA
	            		sensor_set_wakeup_level_suspend(1);
			#endif
			
			cpu_sleep_wakeup(SUSPEND_MODE, PM_WAKEUP_CORE | PM_WAKEUP_PAD, 0);

			usb_resume_host();
			need_wake_up_host_flag =1;
		}
	}
	else
	{
		tick_host_start_power_off=clock_time()|1;
		allow_suspend=0;
		
		need_wake_up_host_flag=0;
		tick_resume_host=0;
	}
}

void usb_mode_loop()
{
	usb_handle_irq();
	
	ui_usb_mode();
	
	device_led_process();

	usb_proc_suspend();
}

_attribute_ram_code_ int main(void)     //must run in ramcode
{
    	blc_pm_select_internal_32k_crystal();
 
#if(MCU_CORE_TYPE == MCU_CORE_8258)
	cpu_wakeup_init();
#elif(MCU_CORE_TYPE == MCU_CORE_8278)
	cpu_wakeup_init(DCDC_MODE, EXTERNAL_XTAL_24M);
#endif

    	deepRetWakeUp = pm_is_MCU_deepRetentionWakeup();

    	gpio_init(!deepRetWakeUp);    		//analog resistance will keep available in deepSleep mode, so no need initialize again

	switch_type_init();

	fun_mode = flash_dev_info.mode;
	
      	report_rate = analog_read(DEEP_ANA_REG2);	//default 0x0f
      	if(report_rate == 0x0f){
		report_rate = 8;
	}
	
	if(((report_rate < 3) && (fun_mode == RF_2M_2P4G_MODE)) || (fun_mode == USB_MODE)){
		clock_init(SYS_CLK_48M_Crystal);
		clock_sys_hz = 48000000;
	}
	else{
		clock_init(SYS_CLK_16M_Crystal);
	}

	u32 bin_size = 0;
	flash_read_page(0x18, 4, (u8 *)&bin_size);
	flash_read_page((bin_size - 4), 4, (u8 *)&output_dev_info.bin_crc);

	if (!deepRetWakeUp)
	{
		deep_flag = analog_read(DEEP_ANA_REG0);
		ana_reg1_aaa = analog_read(USED_DEEP_ANA_REG1);
		pair_flag = ana_reg1_aaa & PAIR_ANA_FLG;

		user_config_ykq();

		dev_info_idx = flash_info_load_aaa(CFG_DEVICE_MODE_ADDR, (u8 *)&flash_dev_info.dongle_id, SAVE_MAX_IN_FLASH);
		if (dev_info_idx < 0)
		{
			flash_dev_info.mode = RF_2M_2P4G_MODE;
			flash_dev_info.mast_id = 0;
			flash_dev_info.slave_mac_addr[0] = 0;
			flash_dev_info.slave_mac_addr[1] = 0;
			flash_dev_info.slave_mac_addr[2] = 0;
			flash_dev_info.slave_mac_addr[3] = 0;
			flash_dev_info.dongle_id = 0;
		}

		blc_app_loadCustomizedParameters();  //load customized freq_offset cap value
 
#if (BLE_SNIFF_DEBUG)
		flash_dev_info.mode = RF_1M_BLE_MODE;
#endif

#if DPI_SAVE_FLASH
		flash_dpi_info.idx = flash_info_load_aaa((MOUSE_DPI_ADDR), (u8 *)&flash_dpi_info.bt_dpi, 6);
        	if (flash_dpi_info.idx < 0)
#endif
        	{
            		flash_dpi_info.bt_dpi = 1;
#if DPI_SAVE_FLASH
			flash_dpi_info.d24g_dpi = 1;
#endif
			flash_dpi_info.usb_dpi = 1;	
        	}
	}

	hw_init();

	if (deep_flag == POWER_ON_ANA_AAA)
	{
		power_on_tick = clock_time() | 1;
#if EMI_TEST_FUN_ENABLE_AAA
		emi_process();
#endif
	}

#if (BATT_CHECK_ENABLE)					//battery check must do before OTA relative operation
	//user_batt_check_init();
#endif

	if (fun_mode == RF_1M_BLE_MODE)		//ble init
	{
		rf_drv_init(RF_MODE_BLE_1M);

		if (deepRetWakeUp)
		{
			user_init_deepRetn();
		}
		else
		{
			user_init_normal();
		}
	}
	else if(fun_mode == RF_2M_2P4G_MODE)	// 2.4g init
	{
		rf_drv_private_2m_init();
		
		if (!deepRetWakeUp)
		{
			d24_user_init();
		}
	}
	else if(fun_mode == USB_MODE)		// usb init
	{ 
		if(report_rate == 1)		// 1k report rate
		{
			timer1_set_mode(TIMER_MODE_SYSCLK,0, 900 * CLOCK_SYS_CLOCK_1US);
		}
		else
		{
			timer1_set_mode(TIMER_MODE_SYSCLK,0, report_rate * CLOCK_SYS_CLOCK_1MS);
		}
	
	      	timer_start(TIMER1);	
		usb_user_init();
	}

    	blc_pm_setDeepsleepRetentionType(DEEPSLEEP_MODE_RET_SRAM_LOW32K);
	irq_enable();

	while (1)
	{
#if (MODULE_WATCHDOG_ENABLE)
		wd_clear(); //clear watch dog
#endif

#if DEBUG_TOGGLE_GPIO_ENABLE
		debug_loop_toggle();
#endif

		if (fun_mode == RF_1M_BLE_MODE)			// ble mode  main_loop
		{
			main_loop();
		}
		else if(fun_mode == RF_2M_2P4G_MODE)	// 2.4g mode  main_loop
		{
			d24_main_loop();
		}
		else if(fun_mode == USB_MODE)			// usb mode  main_loop
		{
			usb_mode_loop();
		}
	
#if (BATT_CHECK_ENABLE)
		user_batt_check_proc();
#endif

        	last_btn_value = btn_value;

#if UART_PRINT_DEBUG_ENABLE
		_attribute_data_retention_user  static u32 tick = 0;
		if (clock_time_exceed(tick, 3000000))
		{
			tick = clock_time();
			// my_printf_aaa("poll mode=%x,ble_status=%x,fifoNum=%x,interval=%x,latency=%x,timeout=%x\r\n", fun_mode, ble_status_aaa, blc_ll_getTxFifoNumber(), bls_ll_getConnectionInterval(), bls_ll_getConnectionLatency(), bls_ll_getConnectionTimeout());
		}
		my_printf_aaa("fifoNum=%x\r\n",  blc_ll_getTxFifoNumber());
#endif
	}
}

